/*
 * This file is part of faultd.
 *
 * Copyright © 2017 Samsung Electronics
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef FAULTD_EVENT_PROCESSOR_H
#define FAULTD_EVENT_PROCESSOR_H

#include "event.h"
#include "notify_queue.h"
#include "list.h"

struct faultd_event_handler {
	char *name;
	/* Would you like to handle this? */
	int (*event_match)(struct faultd_event_handler *handler,
					   struct faultd_event *ev);

	/* Called each time when there is sth in a queue */
	int (*handle_event)(struct faultd_event_handler *handler);

	/* Initialized internally by FW */
	struct nqueue_head event_queue;
	struct sd_event_source *event_queue_sd_source;

	/* has to be initialized */
	struct list_head node;
};

int event_processor_report_event(struct faultd_event *ev);

int event_processor_handler_register(struct faultd_event_handler *handler);
void event_processor_handler_unregister(struct faultd_event_handler *handler);

#define FAULTD_EVENT_HANDLER_REGISTER(EVENT_HANDLER, NAME, TYPE)	    \
static int event_handler_ ##NAME## _init(struct faultd_module *module,  \
										 struct faultd_config *config,	\
										 sd_event *event_loop)			\
{																		\
    return event_processor_handler_register(&EVENT_HANDLER);			\
}																		\
																		\
static void event_handler_ ##NAME## _cleanup(struct faultd_module *module) \
{																		\
    event_processor_handler_unregister(&EVENT_HANDLER);					\
}																		\
																		\
static struct faultd_module event_handler_ ##NAME## _module = {			\
    .name = #NAME,														\
    .type = TYPE,														\
    .init = event_handler_ ##NAME## _init,								\
    .cleanup = event_handler_ ##NAME## _cleanup,						\
    .node = LIST_HEAD_INIT(event_handler_ ##NAME## _module.node),		\
};																		\
																		\
FAULTD_MODULE_REGISTER(&event_handler_ ##NAME## _module)

#endif /* FAULTD_EVENT_PROCESSOR_H */
