/*
 * This file is a part of fauld.
 *
 * Copyright © 2017 Samsung Electronics
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "database.h"

struct nop_adapter {
	struct faultd_database_adapter database_adapter;
};

#define to_nop_adapter(MOD)				\
	container_of(MOD, struct nop_adapter, database_adapter)


static int init_nop_adapter(struct faultd_module *module,
							struct faultd_config *config,
							sd_event *event_loop)
{
	faultd_set_database_adapter(to_database_adapter(module));
	return 0;
}

static void cleanup_nop_adapter(struct faultd_module *module)
{
}

static int nop_store(struct faultd_database_adapter *adapter,
					 struct faultd_object *obj, faultd_oid_t *oid)
{
	return 0;
}

static int nop_get_by_oid(struct faultd_database_adapter *adapter,
						  faultd_oid_t *oid, struct faultd_object *result)
{
	return -ENOENT;
}

static int nop_load(struct faultd_database_adapter *adapter,
					struct faultd_object *query,
					struct faultd_object *result, uint32_t *nr)
{
	*nr = 0;
	return 0;
}

static int nop_new_oid(faultd_oid_t *oid)
{
	memset(oid, 0, sizeof(*oid));
	return 0;
}

static struct nop_adapter nop_adapter = {
	.database_adapter = {
		.module = {
			.name = "nop_dbadapter",
			.type = FAULTD_MODULE_TYPE_DBADAPTER,
			.init = init_nop_adapter,
			.cleanup = cleanup_nop_adapter,
			.node = LIST_HEAD_INIT(nop_adapter.database_adapter.module.node),
		},
		.store = nop_store,
		.get_by_oid = nop_get_by_oid,
		.load = nop_load,
		.new_oid = nop_new_oid,
	}
};

FAULTD_MODULE_REGISTER(&nop_adapter.database_adapter.module);
