/*
 * This file is part of faultd.
 *
 * Copyright © 2017 Samsung Electronics
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef FAULTD_ACTION_EXECUTED_EVENT_H
#define FAULTD_ACTION_EXECUTED_EVENT_H

#include <time.h>

#include "action.h"
#include "event.h"
#include "common.h"

#define ACTION_EXECUTED_EVENT_ID "action_executed"
#define AE_EV_REASON "reason"
#define AE_EV_ACTION "act"
#define AE_EV_ACTION_IMPL "actimpl"
#define AE_EV_ACTION_LOG "actlog"
#define AE_EV_RESULT "res"

struct action_executed_event {
	struct faultd_event event;
	struct faultd_event *reason;
	char *action;
	char *action_impl;
	struct faultd_object *action_log;
	int result;
};

struct ae_event_data {
	/* This is referenced by ae event */
	struct faultd_event *reason;
	/* Strings are deep copied */
	char *action;
	char *action_impl;
	/* Action log is shallow copied */
	struct faultd_object *action_log;
	int result;
};

#define to_action_executed_event(EVENT)						\
	container_of(EVENT, struct action_executed_event, event)

#endif /* FAULTD_ACTION_EXECUTED_EVENT_H */
