#include "system_booted_event.h"
#include "helpers.h"

#define TESTED_MOD system_booted_event

DECLARE_TEST_ENTRY_POINT(TESTED_MOD, sb_event_serialize, void);

static struct faultd_event *alloc_event()
{
	struct system_booted_event *sb_ev;

	sb_ev = calloc(1, sizeof(*sb_ev));
	assert_non_null(sb_ev);

	fill_event(&sb_ev->event);
	sb_ev->prev_boot_id = generate_uuid();

	return &sb_ev->event;
}

static void generate_test_attrs(struct serialize_test_data *td)
{
	struct system_booted_event *sb_ev = to_system_booted_event(td->ev);
	struct serialized_element serialized_attrs[] = {
		S_ELEMENT_EVENT(sb_ev->event),
		S_ELEMENT("prev_boot_id", TYPE_UUID, &sb_ev->prev_boot_id)
	};

	td->test_data = serialized_elems_dup(serialized_attrs,
										 ARRAY_SIZE(serialized_attrs));
	td->size = ARRAY_SIZE(serialized_attrs);
}

static int setup(void **state)
{
	return setup_event_serialization(state, alloc_event, generate_test_attrs,
									 (serialize_func_t)CALL_TEST_ENTRY_POINT(
										 TESTED_MOD, sb_event_serialize));
}

static int teardown(void **state)
{
	struct serialize_test_data *td = *state;
	struct system_booted_event *sb_ev = to_system_booted_event(td->ev);

	free_event(&sb_ev->event);
	free(sb_ev);
	serialized_elems_free(td->test_data, td->size);
	free(td);
	return 0;
}

FAULTD_TEST_GROUP(
	FAULTD_TEST_CASE_NST("sb_ev_serialize_test", test_event_serialization,
						 setup, teardown)
)
